////////////////////////////////////////////////////////////////////////////////
// Name:       Rnt_LoRa_Arduino_Sender.ino                                    //
//             Simpele sketch die LoRa data verzendt en dit in Oled laat zien //
// http://robotigs.nl/robots/includes/parts.php?idpart=253                    //
// Created by: HARB rboek2@gmail.com februar 2021 GPL copyrights              //
// Original:   Modified from the examples of the Arduino LoRa library         //
//             More resources: https://randomnerdtutorials.com                //
// Platform:   TTGO T-Beam V1.0                                               //
////////////////////////////////////////////////////////////////////////////////
// As outputs the following modules are mounted:                              //
// - Oled schermpje ,monochroom 128x64                                        //
//            http://robotigs.nl/robots/includes/parts.php?idpart=113         //
//                                                                            //
// As inputs the following modules are mounted:                               //
//                                                                            //
// For communications and statistics are mounted:                             //
// - Standard Serial Monitor output                                           //
//            http://robotigs.nl/robots/includes/parts.php?idpart=43          //
////////////////////////////////////////////////////////////////////////////////

  //Define the needed header files for the precompiler, no charge if not used --
  #include <SPI.h>                              //Standaard geinstalleerd in IDE
  #include <Wire.h>                             //Standaard geinstalleerd in IDE
  #include <Adafruit_GFX.h>      //Standaard meegeleverd met IDE, zelf activeren
  #include <Adafruit_SSD1306.h>  //Standaard meegeleverd met IDE, zelf activeren
  #include <LoRa.h>             // https://github.com/sandeepmistry/arduino-LoRa

  //Define PINS ----------------------------------------------------------------
  #define ss 18          //5 Define the pins used by the transceiver module LoRa
  #define rst 23                                                       //14 LoRa
  #define dio0 26                                                       //2 LoRa
  #define OLED_RESET    -1                   //Reset pin# or -1 if not available

  //Define VARIABLE EEPROM -----------------------------------------------------
  //Define VARIABLE SETTINGS ---------------------------------------------------
  #define SCREEN_WIDTH 128                       //OLED display width, in pixels
  #define SCREEN_HEIGHT 64                      //OLED display height, in pixels
  //Define VARIABLES -----------------------------------------------------------
  int counter = 0;           //Enkel voor de voorbeeld data voor de oled display
  String counterStr = "0";   //Enkel voor de voorbeeld data voor de oled display
  //Initialize OBJECTS ---------------------------------------------------------
  Adafruit_SSD1306 display(SCREEN_WIDTH, SCREEN_HEIGHT, &Wire, OLED_RESET);
//END OF PRECOMPILER OPTIONS ---------------------------------------------------



void setup() { //Setup runs once ***********************************************
  Serial.begin(115200);       //Nothing more needed for the Serial Monitor RS232
  while (!Serial);                //Wacht totdat de seriele poort is geactiveerd
  Serial.println("LoRa Sender");

  //Start Oled display ---------------------------------------------------------
  if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C)) { //Generate Vdc intr from 3.3V
    Serial.println(F("SSD1306 allocation failed"));    //Oeps, hij doet het niet
    for(;;);                                       //Don't proceed, loop forever
  } //End of if(!display.begin(SSD1306_SWITCHCAPVCC, 0x3C))
  display.clearDisplay();                                     //Clear the buffer
  display.setTextSize(2);                           //1=onleesbaar klein 2=groot
  display.setTextColor(WHITE);                                 //Draw white text

  //Start LoRa -----------------------------------------------------------------
  LoRa.setPins(ss, rst, dio0);                   //Setup LoRa transceiver module
  while (!LoRa.begin(866E6)) {     //433E6=Asia 866E6=Europe 915E6=North America
    Serial.println(".");
    delay(500);
  }  //Sync word assures you dont get LoRa messages from other LoRa transceivers
  LoRa.setSyncWord(0xF3);   //Change(0xF3) to match receiver. Ranges from 0-0xFF
  Serial.println("LoRa Initializing OK!");
  //Test hardware and software -------------------------------------------------
  LoRa.dumpRegisters(Serial);           //Dump alle LoRa registers zonder uitleg
} //End of setup ---------------------------------------------------------------



void loop() { //KEEP ON RUNNING THIS LOOP FOREVER  *****************************
  Serial.print("Sending packet: ");
  Serial.println(counter);
  counterStr = String(counter, DEC);                  //Converts to decimal base
  display.clearDisplay();                                     //Clear the buffer
  display.setCursor(0,0);                             //Start at top-left corner
  display.println("LoRa send");
  display.println(" ");
  display.println(counterStr);
  display.display();                     //Show the display buffer on the screen

  //Send LoRa packet to receiver
  LoRa.beginPacket();
  LoRa.print("hello ");
  LoRa.print(counter);
  LoRa.endPacket();

  counter++;
  delay(10000);
} //End of void loop() ----------------------- KEEP ON RUNNING THIS LOOP FOREVER
