////////////////////////////////////////////////////////////////////////////////
// Naam:       Rnt_LoRaGateway.ino                                            //
//             http://robotigs.com/robots/includes/bots.php?idbot=32          //
//                                                                            //
// Created by: HARB rboek2@gmail.com december 2020 GPL copyrights             //
// Platform:   TTGO T-Beam V1.1                                               //
////////////////////////////////////////////////////////////////////////////////
// As outputs the following modules are mounted:                              //
//                                                                            //
// As inputs the following modules are mounted:                               //
//                                                                            //
// For communications and statistics are mounted:                             //
// - Standard Serial Monitor output                                           //
//            http://robotigs.nl/robots/includes/parts.php?idpart=43          //
////////////////////////////////////////////////////////////////////////////////

// SET PRECOMPILER OPTIONS *****************************************************
  //Define the needed header files for the precompiler, no charge if not used --
  #include <SPI.h>                              //Standaard geinstalleerd in IDE
  #include <Wire.h>                             //Standaard geinstalleerd in IDE
  #include <LoRa.h>             // https://github.com/sandeepmistry/arduino-LoRa
  #include <WiFi.h>                             //Standaard geinstalleerd in IDE
  #include <PubSubClient.h> //Niet100% https://github.com/knolleary/pubsubclient
  #include "SSD1306Wire.h"  //https://github.com/ThingPulse/esp8266-oled-ssd1306
  #include "OLEDDisplayUi.h"//https://github.com/ThingPulse/esp8266-oled-ssd1306
  
  //Define PINS ----------------------------------------------------------------
  #define ss            18        //Define the pins used by the LoRa transceiver
  #define rst           23        //Define the pins used by the LoRa transceiver
  #define dio0          26        //Define the pins used by the LoRa transceiver

  //Initialize OBJECTS ---------------------------------------------------------
  WiFiClient espClient;                                   //Create a WiFi object
  PubSubClient client(espClient);                         //Create a MQTT object
  SSD1306Wire  display(0x3c, 21, 22);             //T-Beam V1.0 => SDA=21 SCL=22
  OLEDDisplayUi ui ( &display );          //Maak een object voor de oled display

  //Define VARIABLE SETTINGS ---------------------------------------------------
  const char* ssid        = "H369AAA781A";            //Inloggen op WiFi netwerk
  const char* password    = "DD7D5CAAA3DE";  //Verwijder paswoord bij publicatie
  const char* mqtt_server = "192.168.2.24";      //Adres van Raspberry Pi server
  int         frameCount  = 1;    //Aantal frames dat meedoet in de oled display

  //Define EEPROM variables ----------------------------------------------------
  //Define DATABASE VARIABLES --------------------------------------------------

  //Define variables -----------------------------------------------------------
  int         counter = 0;  //Telt het aantal keren dat we LoRa hebben verzonden
  String      counterStr = "0";           //Enkel omzetting voor de oled display
  IPAddress   ipa;                             //WiFi IP adres gekregen van DHCP
  String      AdresStr   = "";       //WiFi IP adres gekregen van DHCP voor oled
  long        lastMsg = 0;                                     //Nodig voor MQTT
  char        payLoad[50];                                     //Nodig voor MQTT
  char        topic[50];                                       //Nodig voor MQTT
  String      tmpTopic;                                        //Nodig voor MQTT
  bool        valid = false;                                   //Nodig voor MQTT

  int rssi;                     //Initialize variables to get and save LoRa data
  String loRaMessage;
  String LoRaData;
  String temperature;
  String moisture;
  String conductivity;
  String light;
  String battery;
  String deviceID;
//END OF PRECOMPILER OPTIONS ---------------------------------------------------


//Setup van de oled display ****************************************************
  void frame(OLEDDisplay *display, OLEDDisplayUiState* state, int16_t x, int16_t y) {
    display->setFont(ArialMT_Plain_16);   //Het middelgrote standaard lettertype
    display->drawString(0 + x, 0 + y, "LoRa Gateway");         //Titel programma
    display->drawString(0 + x, 18 + y, AdresStr);            //Ons WiFi IP adres
    display->setFont(ArialMT_Plain_10);        //Het kleine standaard lettertype
    display->drawString(0 + x, 38 + y, counterStr);               //NOG INVULLEN
    display->drawString(0 + x, 50 + y, "192.168.2.34");           //NOG INVULLEN
  } //End of void frame(OLEDDisplay *display
  FrameCallback frames[] = {frame};                //Frames zijn de single views
//End of setup van de oled display ---------------------------------------------


void setup() { //Setup runs once ***********************************************
  Serial.begin(115200);    //Dit is alles wat nodig is voor Serial Monitor RS232
  while (!Serial);                //Wacht totdat de seriele poort is geactiveerd
  Serial.println("LoRa gateway setup\n");

  //Start objects --------------------------------------------------------------
  start_oled();                                    //Maak alle oled instellingen
  start_wifi();                                   //Verbind met een WiFi netwerk
  Serial.print("Bezig met MQTT setup: ");
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
  Serial.println("OK");
  
  Serial.print("Bezig met LoRa setup: ");
  LoRa.setPins(ss, rst, dio0);                   //Setup LoRa transceiver module
  while (!LoRa.begin(866E6)) {   //866E6=Europe, 433E6=Asia, 915E6=North America
    Serial.print("Oeps ");                   //Enkel als het niet in 1 keer lukt
    delay(500);                           //Als het moet dan wachten we nog even
  } //End of while (!LoRa.begin)
  LoRa.setSyncWord(0xF3);      //Change sync word (0-0xFF) to match the receiver
  LoRa.setTxPower(20);                            //Zendvermogen:2-20 default:17
  Serial.println("OK");
} //End of setup ---------------------------------------------------------------








void loop() { //KEEP ON RUNNING THIS LOOP FOREVER  *****************************
  client.loop();  //Spuug resterende data naar de MQTT server en bekijk callback
  int packetSize = LoRa.parsePacket();      //Test of LoRa data wordt aangeboden
  if (packetSize) {                    //Er wordt kennelijk LoRa data aangeboden
      getLoRaData();                         //Lees de data en splits in Strings
      sendMQTT();                           //Verzend data indien valid per MQTT
      int remainingTimeBudget = ui.update();     //Beeldscherm wacht op callback
      counter++;
      counterStr = String(counter);
  } //End of if (packetSize)            Klaar met de verwerking van de LoRa data
} //End of void loop() ----------------------- KEEP ON RUNNING THIS LOOP FOREVER







////////////////////////////////////////////////////////////////////////////////
void sendMQTT() { //Verzend data indien valid per MQTT *************************
  if (valid == true) {           //Indien true worden de data per MQTT verzonden
    LoRaData.toCharArray(payLoad, sizeof(LoRaData));
    if (!client.connected()) {                            //Keep MQTT coneected
       reconnect();                                   //Verbind met MQTT server
    }                                              //Nu zijn we zeker verbonden

    tmpTopic = "/2802ZS9/LoRaGateway/Kas/4in1/" + deviceID + "/temperatuur/";
    tmpTopic.toCharArray(topic, 50);
    temperature.toCharArray(payLoad, sizeof(temperature));
    client.publish(topic, payLoad);

    tmpTopic = "/2802ZS9/LoRaGateway/Kas/4in1/" + deviceID + "/grondvochtigheid/";
    tmpTopic.toCharArray(topic, 50);
    moisture.toCharArray(payLoad, sizeof(moisture));
    client.publish(topic, payLoad);

    tmpTopic = "/2802ZS9/LoRaGateway/Kas/4in1/" + deviceID + "/geleiding/";
    tmpTopic.toCharArray(topic, 50);
    conductivity.toCharArray(payLoad, sizeof(conductivity));
    client.publish(topic, payLoad);

    tmpTopic = "/2802ZS9/LoRaGateway/Kas/4in1/" + deviceID + "/lichtkracht/";
    tmpTopic.toCharArray(topic, 50);
    light.toCharArray(payLoad, sizeof(light));
    client.publish(topic, payLoad);

    tmpTopic = "/2802ZS9/LoRaGateway/Kas/4in1/" + deviceID + "/batterij/";
    tmpTopic.toCharArray(topic, 50);
    battery.toCharArray(payLoad, sizeof(battery));
    client.publish(topic, payLoad);
    Serial.println("MQTT send");                //Send LoRa data to MQTT server
  } else {
    Serial.println("\n LoRa data niet valid dus geen MQTT verzonden");  //RS232    
  }
} //End of sendMQTT() ----------------------- KEEP ON RUNNING THIS LOOP FOREVER






void getLoRaData() { //Read LoRa packet and get the sensor readings ************
  valid = true;                  //Indien true worden de data per MQTT verzonden
  while (LoRa.available()) {                                       //Read packet
    LoRaData = LoRa.readString();
  }
  Serial.print("\n"); 
  Serial.println(LoRaData);
  if (LoRaData.length()<30){//Vermijd foutsituaties waarin deze string langer is
    int pos1 = LoRaData.indexOf('/');                             //Get deviceID
    int pos2 = LoRaData.indexOf('&');
    int pos3 = LoRaData.indexOf('#');
    int pos4 = LoRaData.indexOf('+');
    deviceID     = LoRaData.substring(0, 4);
    temperature  = LoRaData.substring(5, pos1);
    moisture     = LoRaData.substring(pos1+1, pos2);
    conductivity = LoRaData.substring(pos2+1, pos3);
    light        = LoRaData.substring(pos3+1, pos4);
    battery      = LoRaData.substring(pos4+1, LoRaData.length());
    Serial.print("DeviceId=");   
    Serial.print(deviceID);   
    Serial.print("-Temperatuur=");   
    Serial.print(temperature);   
    Serial.print("-Moisture=");   
    Serial.print(moisture);   
    Serial.print("-Conductivity=");   
    Serial.print(conductivity);   
    Serial.print("-light=");   
    Serial.print(light);   
    Serial.print("-"); 
    Serial.print("-battery=");   
    Serial.print(battery);   
    Serial.print("-");
  } else {
    valid = false;                //Indien true worden de data per MQTT verzonden  
  }
  rssi = LoRa.packetRssi();                                           //Get RSSIN
  Serial.print("LoRaRSSI=");    
  Serial.println(rssi);
}




////////////////////////////////////////////////////////////////////////////////
void callback(char* topic, byte* message, unsigned int length) {
  Serial.print("Message arrived on topic: ");
  Serial.print(topic);
  Serial.print(". Message: ");
  String messageTemp;

  for (int i = 0; i < length; i++) {
    Serial.print((char)message[i]);
    messageTemp += (char)message[i];
  }
  Serial.println();
  if (String(topic) == "esp32/output") {
    Serial.print("Changing output to ");
    if(messageTemp == "on"){
      Serial.println("on");
    }
    else if(messageTemp == "off"){
      Serial.println("off");
    }
  }
}

void reconnect() {   // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("Attempting MQTT connection...");
    if (client.connect("ESP8266Client")) {     // Attempt to connect
      Serial.print("connected ");
      //client.subscribe("/esp32/output");       // Subscribe
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds ");
      delay(5000);       // Wait 5 seconds before retrying
    }
  }
}




void start_oled() { //Maak alle oled instellingen ******************************
  ui.setTargetFPS(1);  //Zo langzaam mogelijk des te meer tijd voor LoRa en Wifi
  ui.disableAutoTransition();          //Er hoeft maar 1 frame te worden getoond
  ui.disableAllIndicators();               //Dus ook geen frame indicators nodig
  ui.setFrames(frames, frameCount);                  //Minimaal 1 frame is nodig
  ui.init();                     //Initialising the UI will init the display too
  display.flipScreenVertically();                  //Nodig ivm contructie T-Beam
} //End of start_oled() Maak alle oled instellingen ----------------------------


void start_wifi() { //Verbind met een WiFi netwerk *****************************
  counter = 1;                     //Tel het aantal errors tijdens het verbinden
  WiFi.begin(ssid, password);               //Verbind met het ingestelde netwerk
  while (WiFi.status() != WL_CONNECTED) {                //Test of het is gelukt
    AdresStr = "WiFi error ";      //Als het niet is gelukt geven we een melding
    AdresStr += counter;          //We vertellen ook hoe vaak het niet is gelukt
    int remainingTimeBudget = ui.update();       //Beeldscherm wacht op callback
    delay(500);                      //Wacht even totdat we het opnieuw proberen
  } //End of while (WiFi.status() != WL_CONNECTED)         Test of het is gelukt
  ipa = WiFi.localIP();          //Lees het door DHCP server toegewezen IP adres
  AdresStr = "";                            //Vorm dit IP adres om in een String
  AdresStr += (IPAddress)ipa[0];        //De oled bibliotheek lust enkel Strings
  AdresStr += ":";
  AdresStr += (IPAddress)ipa[1]; 
  AdresStr += ":";
  AdresStr += (IPAddress)ipa[2]; 
  AdresStr += ":";
  AdresStr += (IPAddress)ipa[3];
  int remainingTimeBudget = ui.update();         //Beeldscherm wacht op callback
  remainingTimeBudget = ui.update();             //Beeldscherm wacht op callback
} //End of start_wifi() Verbind met een WiFi netwerk ---------------------------


////////////////////////////////////////////////////////////////////////////////
// PIN ALLOCATIONS TABLE HELTEC WIFILORA 32 V1                                //
// GPIO - Function    - External Connection                              FUNC //
//                                                                            //
//    0 - Push button - Internally connected                          GPO PPG //
//    2 - ADC2_2      - HSPI-WP ONBOARD LED                               LED //
//    4 - ADC2_0      - HSPI-HD TOUCH0                               GPIO ADC //
//    5 - LoRa_Sck    - VSPI-CSO Input is LoRa_Sck                    GPO SPI //
//   12 - ADC2_5      - TOUCH5                                       GPIO ADC //
//   13 - ADC2_4      - TOUCH4                                       GPIO ADC //
//   15 - ADC2_3      - HSPI-CSO TOUCH3                              GPIO ADC //
//   17 - ADC2_4      - U2_TXD                                       GPIO TXD //
//   18 - LoRa_Cs     - VSPI-CLK Input is LoRa_Cs                     GPO SPI //
//   19 - LoRa_MiSO   - VSPI-Q Input is LoRa_MiSo U0_CTS              GPO SPI //
//   21 - SDA         - Input is Vext control                         GPO TWI //
//   22 - SCL         - VSPI-WP U0_RTS                               GPIO TWI //
//   23 - V_SPI_D     -                                               GPO SPI //
//   32 - XTAL32      - INPUT ONLY                                        ADC //
//   33 - XTAL32      - INPUT ONLY                                        ADC //
//   36 - ADC1_0      - INPUT ONLY                                        ADC //
//   37 - ADC1_1      - INPUT ONLY                                        ADC //
//   38 - ADC1_2      - INPUT ONLY                                        ADC //
//   39 - ADC1_3      - INPUT ONLY                                        ADC //
////////////////////////////////////////////////////////////////////////////////
