////////////////////////////////////////////////////////////////////////////////
// Naam:       Rnt_LoRaNode.ino                                               //
//             http://robotigs.com/robots/includes/bots.php?idbot=32          //
//                                                                            //
// Created by: HARB rboek2@gmail.com december 2020 GPL copyrights             //
// Platform:   TTGO T-Beam V1.1                                               //
////////////////////////////////////////////////////////////////////////////////
// As outputs the following modules are mounted:                              //
//                                                                            //
// As inputs the following modules are mounted:                               //
//                                                                            //
// For communications and statistics are mounted:                             //
// - Standard Serial Monitor output                                           //
//            http://robotigs.nl/robots/includes/parts.php?idpart=43          //
////////////////////////////////////////////////////////////////////////////////

// SET PRECOMPILER OPTIONS *****************************************************
  //Define the needed header files for the precompiler, no charge if not used --
  #include <SPI.h>                              //Standaard geinstalleerd in IDE
  #include <LoRa.h>             // https://github.com/sandeepmistry/arduino-LoRa
  #include <BLEDevice.h>                        //Standaard geinstalleerd in IDE

  //Define PINS ----------------------------------------------------------------
  #define ss            18      //Define the pins used by the transceiver module
  #define rst           23      //Define the pins used by the transceiver module
  #define dio0          26      //Define the pins used by the transceiver module

  //Define CONSTANTS -----------------------------------------------------------
  #define SLEEP_DURATION 30 * 60              //Sleep between to runs in seconds
  #define EMERGENCY_HIBERNATE 3 * 60  //Emergency hibernate countdown in seconds
  #define BATTERY_INTERVAL 6 //How often should the battery be read-in run count
  #define RETRY 3                //How often should a device be retried in a run
  //Define EEPROM variables ----------------------------------------------------
  //Define DATABASE VARIABLES --------------------------------------------------

  //Define variables -----------------------------------------------------------
  char* FLORA_DEVICES[] = { "C4:7C:8D:6B:5F:5C", "80:EA:CA:89:4D:16" };
  RTC_DATA_ATTR int bootCount = 0; // boot count used to check if battery status should be read
  static int deviceCount = sizeof FLORA_DEVICES / sizeof FLORA_DEVICES[0]; // device count
  static BLEUUID serviceUUID("00001204-0000-1000-8000-00805f9b34fb"); // the remote service we wish to connect to
  static BLEUUID uuid_version_battery("00001a02-0000-1000-8000-00805f9b34fb"); // the characteristic of the remote service we are interested in
  static BLEUUID uuid_sensor_data("00001a01-0000-1000-8000-00805f9b34fb");
  static BLEUUID uuid_write_mode("00001a00-0000-1000-8000-00805f9b34fb");
  bool   readBattery = true;               //Set if battery status should be read
  
  int    counter = 0;   //Telt het aantal keren dat we LoRa data hebben verzonden
  long   lastMsg = 0;                                        //Counter voor timer
  String LoRaMessage  = "";
  float  temperature  = 0;
  int    moisture     = 0;
  int    light        = 0;
  int    conductivity = 0;
  int    battery      = 0;
  int    readingID    = 1;                                      //Packet counter
  char   tempString[8];
  String DeviceId = "Unknown";
  const String  BASE_TOPIC = "florigs"; 

  //Initialize OBJECTS ---------------------------------------------------------
//END OF PRECOMPILER OPTIONS ---------------------------------------------------



BLEClient* getFloraClient(BLEAddress floraAddress) {
  BLEClient* floraClient = BLEDevice::createClient();
  if (!floraClient->connect(floraAddress)) {
    Serial.println("- Connection failed, skipping");
    return nullptr;
  }
  //Serial.println("- Connection successful");
  return floraClient;
}


BLERemoteService* getFloraService(BLEClient* floraClient) {
  BLERemoteService* floraService = nullptr;
  try { floraService = floraClient->getService(serviceUUID);  }
  catch (...) {  }     // something went wrong
  if (floraService == nullptr) {
    Serial.println("- Failed to find data service");
  } else {
    //Serial.println("- Found data service");
  }
  return floraService;
}





void setup() { //Setup runs once ***********************************************
  Serial.begin(115200);    //Dit is alles wat nodig is voor Serial Monitor RS232
  while (!Serial);                //Wacht totdat de seriele poort is geactiveerd
  Serial.println("\n\nLoRa node setup\n");     //Geef status naar serial monitor

  //Start objects --------------------------------------------------------------
  Serial.print("Bezig met LoRa setup: ");
  LoRa.setPins(ss, rst, dio0);                   //Setup LoRa transceiver module
  while (!LoRa.begin(866E6)) {   //866E6=Europe, 433E6=Asia, 915E6=North America
    Serial.print("Oeps ");                   //Enkel als het niet in 1 keer lukt
    delay(500);                           //Als het moet dan wachten we nog even
  } //End of while (!LoRa.begin)
  LoRa.setSyncWord(0xF3);     // Change sync word (0-0xFF) to match the receiver
  Serial.println("OK");
  //Test hardware and software -------------------------------------------------
  read_4in1_sensors();             //Reads all 4in1 devices and LoRa the results
} //End of setup ---------------------------------------------------------------





void loop() { //KEEP ON RUNNING THIS LOOP FOREVER  *****************************
  long now = millis();
  if (now - lastMsg > 600000) {                                 //60000=1 minuut 
    lastMsg = now;                                      //Reset interval counter
    read_4in1_sensors();           //Reads all 4in1 devices and LoRa the results
    counter++;                                        //Prepare for next message
  } //End of if (now - lastMsg > X)
} //End of void loop() ----------------------- KEEP ON RUNNING THIS LOOP FOREVER






void read_4in1_sensors() { //Reads all 4in1 devices and LoRa the results ********
  bootCount++;     // increase boot count
  //Serial.println("\nInitialize BLE client...");
  BLEDevice::init("4in1");
  BLEDevice::setPower(ESP_PWR_LVL_P7);
  for (int i=0; i<deviceCount; i++) {  // process devices
    int tryCount = 0;
    char* deviceMacAddress = FLORA_DEVICES[i];
    if (deviceMacAddress == "C4:7C:8D:6B:5F:5C"){
      DeviceId = "Ri01";
    }
    if (deviceMacAddress == "80:EA:CA:89:4D:16"){
      DeviceId = "Ro01";
    }
    BLEAddress floraAddress(deviceMacAddress);
    while (tryCount < RETRY) {
      tryCount++;
      if (processFloraDevice(floraAddress, deviceMacAddress, readBattery, tryCount)) {
        break;
      }
      delay(1000);
    }
    delay(500);
    sendReadings();                         //Send a message to the LoRa gateway
    delay(15000);       //Give the gateway some time to process this 4in1 sensor
  }
} //End of void read_4in1_sensors() --------------------------------------------








void sendReadings() { //Send a message to the LoRa gateway *********************
  LoRaMessage = String(DeviceId);
  dtostrf(temperature, 1, 2, tempString);    //Convert the value to a char array
  LoRaMessage = LoRaMessage + " " + String(tempString);
  dtostrf(moisture, 1, 0, tempString);       //Convert the value to a char array
  LoRaMessage = LoRaMessage + "/" + String(tempString);
  dtostrf(light, 1, 0, tempString);          //Convert the value to a char array
  LoRaMessage = LoRaMessage + "&" + String(tempString);
  dtostrf(conductivity, 1, 0, tempString);   //Convert the value to a char array
  LoRaMessage = LoRaMessage + "#" + String(tempString);
  dtostrf(battery, 1, 0, tempString);        //Convert the value to a char array
  LoRaMessage = LoRaMessage + "+" + String(tempString);
    
  Serial.print("Sending ");
  LoRa.beginPacket();                             //Send LoRa packet to receiver
  LoRa.print(LoRaMessage);
  LoRa.endPacket();
  Serial.print(" LoRaMessage: ");      
  Serial.println(LoRaMessage);
  readingID++;
} //End of void sendReadings() -------------------------------------------------












bool forceFloraServiceDataMode(BLERemoteService* floraService) {
  BLERemoteCharacteristic* floraCharacteristic;
  //Serial.println("- Force device in data mode");   // get device mode characteristic, needs to be changed to read data
  floraCharacteristic = nullptr;
  try { floraCharacteristic = floraService->getCharacteristic(uuid_write_mode); }
  catch (...) {  }     // something went wrong
  if (floraCharacteristic == nullptr) {
    Serial.println("-- Failed, skipping device");
    return false;
  }
  uint8_t buf[2] = {0xA0, 0x1F};   // write the magic data
  floraCharacteristic->writeValue(buf, 2, true);
  delay(500);
  return true;
}







bool readFloraDataCharacteristic(BLERemoteService* floraService, String baseTopic) {
  BLERemoteCharacteristic* floraCharacteristic = nullptr;

  //Serial.println("- Access characteristic from device");   // get the main device data characteristic
  try { floraCharacteristic = floraService->getCharacteristic(uuid_sensor_data);  }
  catch (...) { }     // something went wrong
  if (floraCharacteristic == nullptr) {
    Serial.println("-- Failed, skipping device");
    return false;
  }

  //Serial.println("- Read value from characteristic");   // read characteristic value
  std::string value;
  try{ value = floraCharacteristic->readValue();  }
  catch (...) {          // something went wrong
    Serial.println("-- Failed, skipping device"); 
    return false;
  }
  const char *val = value.c_str();
  //Serial.print("Hex: ");
  for (int i = 0; i < 16; i++) {
    //Serial.print((int)val[i], HEX);
    //Serial.print(" ");
  }
  //Serial.println(" ");

  int16_t* temp_raw = (int16_t*)val;
  temperature = (*temp_raw) / ((float)10.0);
  Serial.print("-- Temperatuur: ");
  Serial.println(temperature);

  moisture = val[7];
  Serial.print("-- Moisture: ");
  Serial.println(moisture);

  light = val[3] + val[4] * 256;
  Serial.print("-- Light: ");
  Serial.println(light);
 
  conductivity = val[8] + val[9] * 256;
  Serial.print("-- Conductivity: ");
  Serial.println(conductivity);

  if (temperature > 200) {
    Serial.println("-- Unreasonable values received, skip publish");
    return false;
  }
  return true;
}









bool readFloraBatteryCharacteristic(BLERemoteService* floraService, String baseTopic) {
  BLERemoteCharacteristic* floraCharacteristic = nullptr;
  //Serial.println("- Access battery characteristic from device");   // get the device battery characteristic
  try {
    floraCharacteristic = floraService->getCharacteristic(uuid_version_battery);
  }
  catch (...) {
    // something went wrong
  }
  if (floraCharacteristic == nullptr) {
    Serial.println("-- Failed, skipping battery level");
    return false;
  }

  // read characteristic value
  //Serial.println("- Read value from characteristic");
  std::string value;
  try{
    value = floraCharacteristic->readValue();
  }
  catch (...) {
    // something went wrong
    Serial.println("-- Failed, skipping battery level");
    return false;
  }
  const char *val2 = value.c_str();
  battery = val2[0];

  char buffer[64];

  Serial.print("-- Battery: ");
  Serial.println(battery);
  return true;
}










bool processFloraService(BLERemoteService* floraService, char* deviceMacAddress, bool readBattery) {
  if (!forceFloraServiceDataMode(floraService)) {     //Set device in data mode
    Serial.print(" FOUT in forceFloraServiceDataMode!!! ");
    return false;
  }
  String baseTopic = BASE_TOPIC + "/" + deviceMacAddress + "/";
  bool dataSuccess = readFloraDataCharacteristic(floraService, baseTopic);
  bool batterySuccess = true;
  if (readBattery) {
    batterySuccess = readFloraBatteryCharacteristic(floraService, baseTopic);
  }
  return dataSuccess && batterySuccess;
}






bool processFloraDevice(BLEAddress floraAddress, char* deviceMacAddress, bool getBattery, int tryCount) {
  Serial.print("\nProcessing Flora device at ");
  Serial.print(floraAddress.toString().c_str());
  Serial.print(" (try ");
  Serial.print(tryCount);
  Serial.println(")");
  BLEClient* floraClient = getFloraClient(floraAddress);   // connect to flora ble server
  if (floraClient == nullptr) {
    Serial.print(" FOUT1 ");
    return false;
  }
  BLERemoteService* floraService = getFloraService(floraClient);  // connect data service
  if (floraService == nullptr) {
    floraClient->disconnect();
    Serial.print(" FOUT2 ");
    return false;
  }
  bool success = processFloraService(floraService, deviceMacAddress, getBattery);   // process devices data
  floraClient->disconnect();   // disconnect from device
  //Serial.print(" FOUT3 ");
  return success;
}
